/* node.h
 * libxml++ and this file are copyright (C) 2000 by Ari Johnson, and
 * are covered by the GNU Lesser General Public License, which should be
 * included with libxml++ as the file COPYING.
 */

#ifndef __LIBXMLPP_NODES_NODE_H
#define __LIBXMLPP_NODES_NODE_H

#include <string>
#include <list>
#include <map>
#include <vector>

#include <libxml/tree.h>

namespace xmlpp {

class TextNode;
class Element;
class Attribute;

class Node;
typedef std::vector<Node*> NodeSet;

/** Represents XML Nodes.
 * You should never new or delete Nodes. The Parser will create and manage them for you.
 */
class Node
{
public:
  typedef std::list<Node*> NodeList;
  typedef std::list<Attribute*> AttributeList;

  explicit Node(xmlNode* node);
  virtual ~Node();

  std::string get_name() const;
  int get_line() const;

  bool has_content() const;
  
  // FIXME: this should be reimplemented in two ways:
  // a) using iterators
  // b) returning a heap-allocated buffer containing the
  //    concatenation of all text node content
  //
  // for now only return the first text node (if any)
  TextNode* get_child_content();
  const TextNode* get_child_content() const;

  // FIXME: define that more clearly...
  //
  // if this is a text node, append the content
  // if this is an element node, append a new text node
  TextNode* add_content(const std::string& content = std::string());
  void set_child_content(const std::string& content);
  
  NodeList get_children(const std::string& name = std::string());
  const NodeList get_children(const std::string& name = std::string()) const;

  Element* add_child(const std::string& name);
  // FIXME: how should this work in general (depending on the exact types
  // of parent and child node)
  //   Node* add_child(Node* node);


  /** Remove the child node.
   * @param node The child node to remove. This Node will be deleted and therefore unusable after calling this method.
   */
  void remove_child(Node* node);

  std::string get_path() const;
  NodeSet find(const std::string& xpath) const;

  ///Access the underlying libxml implementation.
  xmlNode* cobj();

  ///Access the underlying libxml implementation.
  const xmlNode* cobj() const;

private:
   xmlNode* _impl;
};

} // namespace xmlpp

#endif //__LIBXMLPP_NODES_NODE_H



